declare type ArgsType<F extends Function> = F extends (...args: infer A) => unknown ? A : [];
declare type ReturnVal<F extends Function> = F extends (...args: ArgsType<F>) => infer R ? R : unknown;
declare type PromiseReturnVal<F extends Function> = ReturnVal<F> extends Promise<unknown> ? F : (...args: ArgsType<F>) => Promise<ReturnVal<F>>;
declare type Nothing = void | null | undefined;
declare type EmptyPipe<F extends Function> = (...args: ArgsType<F>) => ArgsType<F>;
declare type PipeFunctions<F1 extends Function, F2 extends Function> = ReturnVal<F2> extends Nothing ? ReturnVal<F1> extends Nothing ? EmptyPipe<F1> : F1 : (...args: ArgsType<F1>) => ReturnVal<F2>;
declare type ReturnValOrArgs<F extends Function> = ReturnVal<F> extends Nothing ? ArgsType<F> : ReturnVal<F>;
declare type UnPromise<T> = T extends Promise<infer R> ? R : T;
declare type Arrayify<T> = T extends Array<unknown> ? T : T[];
declare type ExtendingPipeFn<F extends Function> = (...args: Arrayify<UnPromise<ReturnValOrArgs<F>>>) => unknown;
declare type AsyncPipeFunction<O extends Function> = PromiseReturnVal<O> & {
    pipe: <N extends ExtendingPipeFn<O>>(fn: N) => AsyncPipeFunction<PipeFunctions<O, N>>;
    then: <N extends ExtendingPipeFn<O>>(fn: N) => AsyncPipeFunction<PipeFunctions<O, N>>;
};
export declare function createPipe<T extends Function>(pipeSrc: T): AsyncPipeFunction<T>;
export declare function createLogBulkPipe<T>(): AsyncPipeFunction<(bulk: T[], bulkIndex: number) => void>;
export declare function logBulk<T>(bulk: T[], bulkIndex: number): void;
export declare function createDelay(timeout: number): () => Promise<void>;
export {};
